/**
 * MyDomparser.java
 * Updated 11-11-2002
 */

package xml;

import java.io.File;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.IOException;

// Xerces parser
import org.apache.xerces.parsers.DOMParser;

// DOM
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Attr;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.DOMException;
import org.w3c.dom.ranges.DocumentRange;

// SAX Exceptions
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
  * DomParser class.
  * Parses an xml file using the Xerces parser and the DOM api.
  * @author Thomas Rowland
  */
public class MyDomParser {

	private static DOMParser parser = null;
	private static PrintStream out = null;

	static final int ELEMENT_NODE 	= 1;
	static final int ATTR_NODE 	= 2;
	static final int TEXT_NODE 	= 3;

	/**
	 * Parses an xml file 
	 * @param uri the location of the xml file
	 * @param os the OutputStream to write the output to
	 */
	public static void parse (String uri, OutputStream os) 
		throws Exception {
			
		try {
                    //InputSource src = new InputSource (is);
                    out = new PrintStream (os);
                    parser = new DOMParser();
                    parser.parse (uri);
                    Document domDocument = parser.getDocument();
                    Element root = domDocument.getDocumentElement ();
                    traverse (root);
                    out.flush ();
                    out.close ();
                } 
		catch (SAXException e) {
                    // get the wrapped exception, if any
		    Exception ex = e.getException ();
		    String msg = null;
		    if (ex != null) {
		    	ex.printStackTrace ();
		    	msg = ex.getMessage ();
		    }
			throw new Exception (
                        "** SAXException:\n" + e.getMessage ());
		}
		catch (IOException e) {
                    throw new Exception (
                    "IOException:\n" + e.getMessage ());
		}
                catch (Exception e) {
                    System.out.println (
                    "Exception:\n" + e.getMessage ());
                }
	}//parse
	
	
	/**
	 * Recursive method which traverses the DOM Document
	 * and outputs the content.
	 * @param elem the root element to begin recursion on
	 */
	public static void traverse (Node elem) {
            try {
                // handle the attributes
		if (elem.hasAttributes ()) {
                    NamedNodeMap attrs = elem.getAttributes ();
                    int alength = attrs.getLength ();
                    for (int i=0; i<alength; i++) {
			Attr attr = (Attr) attrs.item (i);
			out.println (
                            attr.getName () + ":\t" + attr.getValue ());
                    }
		}
                
                // handle the child nodes
                if (elem.hasChildNodes ()) {
                    NodeList children = elem.getChildNodes ();
                    int length = children.getLength ();

                    for (int i=0; i<length; i++) {
                        Node n = children.item (i);
                        String name = n.getNodeName ();

                        if (n.getNodeType () == ELEMENT_NODE) {
                            if (name.equals ("Item"))
                                out.println ("\n");
                            else
                                out.print (name + ":\t");
                            
                            // recurse
                            traverse (n);
                        }
                        else
                        if (n.getNodeType () == TEXT_NODE) {
                            String txt = n.getNodeValue ().trim ();
                            if (! txt.equals ("")) {
                                out.println (txt);
                            }
                        }
                    }

                    return;
                }
            }
            catch (DOMException e) {
		System.out.println (
                "*** DOMException\n" + e.getMessage ());
            }
	}//traverse

}//class